#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: generate_vserver_list.pl, v1.2.0.0 Sun 12 Apr 2009 02:19:23 PM CEST mstauber Exp $
# Copyright 2006-2009 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.

# This constructor / handler uses 'vzlist' to poll extended vserver status.
# Additionally it parses the vserver config files in /etc/sysconfig/vz-scripts/
# The information is then written off into CCE for GUI usage. 

$whatami = "constructor";

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

if ($whatami eq "handler") {
    $cce->connectfd();
}
else {
    $cce->connectuds();
}

# Location of the vzlist-dump:
$vzlistdump = '/tmp/vserver-list';

# Location of vserver config files (directory - with trailing slash):
$vzcfgdir = '/etc/sysconfig/vz-scripts/';

# We are starting sane:
$bad_info_detected = '0';

# Parse existing VPS and shove their data into CCE:
&main;

# If we detect bad info, we will try again. Up to three times.
# Then we give up as something is barfed up badly.
if (($bad_info_detected < '3') && ($bad_info_detected != '0')) {
    &main;
}
else {
    exit 1;
}

# CCE'ize example VPS config files:
&sampleconfigs;

# CCE'ize existing OS-Templates:
&ostemplates;

exit 0;

##
# Subs:
##

sub main {

    # Array of VPSIDs:
    @matrix_vpsids = ();

    # Generate vzlist-dump:
    system("vzlist -a -o vpsid,ip,hostname,status,numproc,kmemsize,kmemsize.m,kmemsize.b,kmemsize.l,kmemsize.f,lockedpages,lockedpages.m,lockedpages.b,lockedpages.l,lockedpages.f,privvmpages,privvmpages.m,privvmpages.b,privvmpages.l,privvmpages.f,shmpages,shmpages.m,shmpages.b,shmpages.l,shmpages.f,numproc,numproc.m,numproc.b,numproc.l,numproc.f,physpages,physpages.m,physpages.b,physpages.l,physpages.f,vmguarpages,vmguarpages.m,vmguarpages.b,vmguarpages.l,vmguarpages.f,oomguarpages,oomguarpages.m,oomguarpages.b,oomguarpages.l,oomguarpages.f,numtcpsock,numtcpsock.m,numtcpsock.b,numtcpsock.l,numtcpsock.f,numflock,numflock.m,numflock.b,numflock.l,numflock.f,numpty,numpty.m,numpty.b,numpty.l,numpty.f,numsiginfo,numsiginfo.m,numsiginfo.b,numsiginfo.l,numsiginfo.f,tcpsndbuf,tcpsndbuf.m,tcpsndbuf.b,tcpsndbuf.l,tcpsndbuf.f,tcprcvbuf,tcprcvbuf.m,tcprcvbuf.b,tcprcvbuf.l,tcprcvbuf.f,othersockbuf,othersockbuf.m,othersockbuf.b,othersockbuf.l,othersockbuf.f,dgramrcvbuf,dgramrcvbuf.m,dgramrcvbuf.b,dgramrcvbuf.l,dgramrcvbuf.f,numothersock,numothersock.m,numothersock.b,numothersock.l,numothersock.f,dcachesize,dcachesize.m,dcachesize.b,dcachesize.l,dcachesize.f,numfile,numfile.m,numfile.b,numfile.l,numfile.f,numiptent,numiptent.m,numiptent.b,numiptent.l,numiptent.f,diskspace,diskspace.s,diskspace.h,diskinodes,diskinodes.s,diskinodes.h,laverage,cpulimit,cpuunits | sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzlistdump");

    # Pull the vzlist-dump into a Matrix:
    open (F, $vzlistdump) || die "Could not open $vzlistdump: $!";
    $a = 0;                             	# Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
    while ($line = <F>) {

        chomp($line);
        next if $line =~ /^\s*$/;               # skip blank lines
        next if $line =~ /^VEID/;               # skip lines starting with VEID
        next if $line =~ /^CTID/;               # skip lines starting with VEID
        $matrix_name = 'VPSID';
        my (@row) = split (/\s+/, $line);
        push (@{$matrix_name}, \@row);

    	# Build an array that contains the VPSIDs:
        if ($VPSID[$a][0]) {                    # Plausibility check. We will not store anything into CCE if 'vpsid' is not set.
            $currentvpsid = $VPSID[$a][0];
	    push (@matrix_vpsids, $VPSID[$a][0]);
        }
        $a++;
    }
    close(F);
    system("rm $vzlistdump");

    # At this point we have all data from VZLIST in a matrix. But there is more data to be had in the config files. We need to get that as well:

    # How many entries do we have in the Matrix?
    $matrix_entries = $#VPSID;

    # How many VPSIDs are in the Array?
    $num_matrix_vpsids = $#matrix_vpsids;

    # Set start of Matrix line:
    $matrix_line = 0;

    foreach $currentvpsid (@matrix_vpsids) {
	&readconfig;                        	# Read missing info from vz config files
        if ($whatami ne "handler") {
        	# We only run an update of the IP address allocation if we're a constructor:
               	&ip_alloc;
        }
	&feedthemonster;            		# Feed info into CCE
	$matrix_line++;
    } 

    ##
    # Get info about the current CPU utilization and the power of the node:
    ##

    # Location of the vzcpu-dump:
    $vzcpudump = '/tmp/vserver-cpuinfo';

    # Generate vzcpu-dump:
    system("vzcpucheck | sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzcpudump");

    if (-e $vzcpudump) {
        $b = 0;
        open(CPUDUMP,"$vzcpudump") || die("Could not find $vzcpudump. $!");
        while ($line = <CPUDUMP>) {
            chomp($line);
            next if $line =~ /^\s*$/; 		# skip blank lines
            if (($line =~ /^Current CPU utilization: (.*)/) && ($b == 0)) {
                $CURRENT_CPU = $1;
            }
            if (($line =~ /^Power of the node: (.*)/) && ($b == 1)) {
                $NODE_CPU = $1;
            }
            $b++;
        }
        close(CPUDUMP);
        system("rm $vzcpudump");
        &cce_update_cpu; 			# Write the info into CCE
    }
}

# There are a few options in config files that vzlist doesn't provide.
# So we need to read them from the respective config files in
# /etc/sysconfig/vz-scripts/*.conf

sub readconfig {
    # Does the config file exist?
    $cfile = "$vzcfgdir$currentvpsid.conf";

    if (-e $cfile) {
    	open(CFILE,"$cfile");
    	while ($line = <CFILE>) {
    	    chomp($line);
    	    next if $line =~ /^\s*$/; 		# skip blank lines

    	    if ($line =~ /^ONBOOT=\"([A-Za-z]\w*)\"/) {
    		$onboot = $1;
		if (($onboot eq 'yes') || ($onboot eq 'Yes')) {
    		    $VPSID[$matrix_line][114] = 'Yes';
		}
		else {
    		    $VPSID[$matrix_line][114] = 'No';
		}
	    }
    	    if ($line =~ /^CPUUNITS=\"(.*)\"/) {
    		$VPSID[$matrix_line][113] = $1;
	    }
	    if ($line =~ /^VE_ROOT=\"(.*)\"/) {
		$VPSID[$matrix_line][115] = $1;
	    }
    	    if ($line =~ /^VE_PRIVATE=\"(.*)\"/) {
        	$VPSID[$matrix_line][116] = $1;
    	    }
    	    if ($line =~ /^OSTEMPLATE=\"(.*)\"/) {
        	$VPSID[$matrix_line][117] = $1;
	    }
    	    if ($line =~ /^ORIGIN_SAMPLE=\"(.*)\"/) {
        	$VPSID[$matrix_line][118] = $1;
	    }
            if ($line =~ /^NAMESERVER=\"(.*)\"/) {
                $nameservers = $1;
                $nameservers =~ s/ /&/g;
                $VPSID[$matrix_line][119] = "&" . $nameservers . "&";
            }
	    if ($line =~ /^QUOTATIME=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][120] = $1;
    	    }
	    $quota = '';
	    if ($line =~ /^DISK_QUOTA=\"(.*)\"/) {
		$quota = $1;
		if (($quota eq 'yes') || ($quota eq 'Yes')) {
    	    	    $VPSID[$matrix_line][121] = 'Yes';
		}
		else {
    	    	    $VPSID[$matrix_line][121] = 'No';
		}
    	    }
	    if (($line =~ /^DISK_QUOTA=(.*)/) && ($quota eq '')){
		$quota = $1;
		if (($quota eq 'yes') || ($quota eq 'Yes')) {
    	    	    $VPSID[$matrix_line][121] = 'Yes';
		}
		else {
    	    	    $VPSID[$matrix_line][121] = 'No';
		}
    	    }

	    if ($line =~ /^QUOTAUGIDLIMIT=\"(.*)\"/) {
    	    	$VPSID[$matrix_line][122] = $1;
    	    }
	    if ($line =~ /^AVNUMPROC=\"(.*):(.*)\"/) {
    	    	$VPSID[$matrix_line][123] = $1;
    	    	$VPSID[$matrix_line][124] = $2;
    	    }

	    # ip_list:
	    if ($line =~ /^IP_ADDRESS=\"(.*)\"/) {
                $ip_list = $1;
                $ip_list =~ s/ /&/g;
		$combined_ip_list = "&" . $ip_list . "&";
		@raw_ip_list =  split('&', $combined_ip_list);
		shift(@raw_ip_list);
		shift(@raw_ip_list);
		$extra_ip_list = join("&", @raw_ip_list);
		if ($extra_ip_list) {
	                $VPSID[$matrix_line][125] = "&" . $extra_ip_list . "&";
		}
	    }
	}
	close(CFILE);

	# NAMESERVER must be of format 'ipAddr', but on new VPS's it may not be set - yet. If it is
	# NOT set, we set it to '127.0.0.1' here:
	if (!$VPSID[$matrix_line][119]) {
    	    $VPSID[$matrix_line][119] = "127.0.0.1";
	}
	# QUOTATIME may not be set. If it's not set, we set it to zero:
        if (!$VPSID[$matrix_line][120]) {
	    $VPSID[$matrix_line][120] = "0";
        }
	# DISK_QUOTA may not be set. Valid entries are "yes" or "no". If it's neither, we default to "no":
        if (($VPSID[$matrix_line][121] ne "Yes") && ($VPSID[$matrix_line][121] ne "No")) {
            $VPSID[$matrix_line][121] = "No";
        }
	# QUOTAUGIDLIMIT may not be set. This is for second level quota (user and group disk quota inside the VPS)
        if (!$VPSID[$matrix_line][122]) {
	    $VPSID[$matrix_line][122] = "10000";
        }
    }
}

# Subroutine that feeds the data into CCE:
sub feedthemonster {
	&validate;
	# This is where we actually populate CCE with the gathered info:

	# Check if the VPS is already present in CCE under the old VPS ID:
        @oids = $cce->find('Vserver', {'vpsid' => $currentvpsid});

	# Check if the VPS is already present under a different VPS ID:
	@moved_oids = $cce->find('Vserver', {'ip' => $VPSID[$matrix_line][1], 'hostname' => $VPSID[$matrix_line][2]});

	# Legend: 
	# $#oids : $#moved_oids : action
	#
	#  0 : 0  : update
	# -1 : 0  : update
	# -1 : -1 : create:

        if (($#oids == -1) && ($#moved_oids == -1)) {

                # Object not yet in CCE. Creating new one:
                ($ok) = $cce->create('Vserver', {
                'vpsid' => $VPSID[$matrix_line][0],
                'ip' => $VPSID[$matrix_line][1],
                'hostname' => $VPSID[$matrix_line][2],
                'timestamp' => time()
                });
		$new_vps_found = '1'; 			# We found a new VPS, one that we have never seen before and which
							# is not yet in CCE. We do not want to risk to populate CCE with ALL
							# the found new data in one go. So let us just set the most basic info
							# and the timestamp. 
							#
							# We also set $new_vps_found = '1' to run feedthemonster() again.
        }
        else {
	        #print "Object already present in CCE. Updating it.\n";
                # find the system object
		$new_vps_found = '0';
		&validate;

		if (($#oids < 0) && ($#moved_oids == -1)) {
                	($sys_oid) = $cce->find('Vserver', {'vpsid' => $currentvpsid});
                	($ok, $sys) = $cce->get($sys_oid);
		}
		else {
                	($sys_oid) = $cce->find('Vserver', {'ip' => $VPSID[$matrix_line][1], 'hostname' => $VPSID[$matrix_line][2]});
                	($ok, $sys) = $cce->get($sys_oid);
		}
		
		# For debugging and logging purpose we do not write all 122 entries into CCE in one go. Instead we break it down
		# into multiple blocks so that we can troubleshoot /var/log/messages easily for data-blocks with values that CCE
		# did not like. 
		
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'vpsid' => $VPSID[$matrix_line][0],
                'ip' => $VPSID[$matrix_line][1],
                'ip_list' => $VPSID[$matrix_line][125],
                'hostname' => $VPSID[$matrix_line][2],
                'status' => $VPSID[$matrix_line][3],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'avnumproc_b' => $VPSID[$matrix_line][123],
                'avnumproc_l' => $VPSID[$matrix_line][124],
                'kmemsize' => $VPSID[$matrix_line][5],
                'kmemsize_m' => $VPSID[$matrix_line][6],
                'kmemsize_b' => $VPSID[$matrix_line][7],
                'kmemsize_l' => $VPSID[$matrix_line][8],
                'kmemsize_f' => $VPSID[$matrix_line][9],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'lockedpages' => $VPSID[$matrix_line][10],
                'lockedpages_m' => $VPSID[$matrix_line][11],
                'lockedpages_b' => $VPSID[$matrix_line][12],
                'lockedpages_l' => $VPSID[$matrix_line][13],
                'lockedpages_f' => $VPSID[$matrix_line][14],
                'privvmpages' => $VPSID[$matrix_line][15],
                'privvmpages_m' => $VPSID[$matrix_line][16],
                'privvmpages_b' => $VPSID[$matrix_line][17],
                'privvmpages_l' => $VPSID[$matrix_line][18],
                'privvmpages_f' => $VPSID[$matrix_line][19],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'shmpages' => $VPSID[$matrix_line][20],
                'shmpages_m' => $VPSID[$matrix_line][21],
                'shmpages_b' => $VPSID[$matrix_line][22],
                'shmpages_l' => $VPSID[$matrix_line][23],
                'shmpages_f' => $VPSID[$matrix_line][24],
                'numproc' => $VPSID[$matrix_line][25],
                'numproc_m' => $VPSID[$matrix_line][26],
                'numproc_b' => $VPSID[$matrix_line][27],
                'numproc_l' => $VPSID[$matrix_line][28],
                'numproc_f' => $VPSID[$matrix_line][29],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'physpages' => $VPSID[$matrix_line][30],
                'physpages_m' => $VPSID[$matrix_line][31],
                'physpages_b' => $VPSID[$matrix_line][32],
                'physpages_l' => $VPSID[$matrix_line][33],
                'physpages_f' => $VPSID[$matrix_line][34],
                'vmguarpages' => $VPSID[$matrix_line][35],
                'vmguarpages_m' => $VPSID[$matrix_line][36],
                'vmguarpages_b' => $VPSID[$matrix_line][37],
                'vmguarpages_l' => $VPSID[$matrix_line][38],
                'vmguarpages_f' => $VPSID[$matrix_line][39],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'oomguarpages' => $VPSID[$matrix_line][40],
                'oomguarpages_m' => $VPSID[$matrix_line][41],
                'oomguarpages_b' => $VPSID[$matrix_line][42],
                'oomguarpages_l' => $VPSID[$matrix_line][43],
                'oomguarpages_f' => $VPSID[$matrix_line][44],
                'numtcpsock' => $VPSID[$matrix_line][45],
                'numtcpsock_m' => $VPSID[$matrix_line][46],
                'numtcpsock_b' => $VPSID[$matrix_line][47],
                'numtcpsock_l' => $VPSID[$matrix_line][48],
                'numtcpsock_f' => $VPSID[$matrix_line][49],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'numflock' => $VPSID[$matrix_line][50],
                'numflock_m' => $VPSID[$matrix_line][51],
                'numflock_b' => $VPSID[$matrix_line][52],
                'numflock_l' => $VPSID[$matrix_line][53],
                'numflock_f' => $VPSID[$matrix_line][54],
                'numpty' => $VPSID[$matrix_line][55],
                'numpty_m' => $VPSID[$matrix_line][56],
                'numpty_b' => $VPSID[$matrix_line][57],
                'numpty_l' => $VPSID[$matrix_line][58],
                'numpty_f' => $VPSID[$matrix_line][59],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'numsiginfo' => $VPSID[$matrix_line][60],
                'numsiginfo_m' => $VPSID[$matrix_line][61],
                'numsiginfo_b' => $VPSID[$matrix_line][62],
                'numsiginfo_l' => $VPSID[$matrix_line][63],
                'numsiginfo_f' => $VPSID[$matrix_line][64],
                'tcpsndbuf' => $VPSID[$matrix_line][65],
                'tcpsndbuf_m' => $VPSID[$matrix_line][66],
                'tcpsndbuf_b' => $VPSID[$matrix_line][67],
                'tcpsndbuf_l' => $VPSID[$matrix_line][68],
                'tcpsndbuf_f' => $VPSID[$matrix_line][69],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'tcprcvbuf' => $VPSID[$matrix_line][70],
                'tcprcvbuf_m' => $VPSID[$matrix_line][71],
                'tcprcvbuf_b' => $VPSID[$matrix_line][72],
                'tcprcvbuf_l' => $VPSID[$matrix_line][73],
                'tcprcvbuf_f' => $VPSID[$matrix_line][74],
                'othersockbuf' => $VPSID[$matrix_line][75],
                'othersockbuf_m' => $VPSID[$matrix_line][76],
                'othersockbuf_b' => $VPSID[$matrix_line][77],
                'othersockbuf_l' => $VPSID[$matrix_line][78],
                'othersockbuf_f' => $VPSID[$matrix_line][79],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'dgramrcvbuf' => $VPSID[$matrix_line][80],
                'dgramrcvbuf_m' => $VPSID[$matrix_line][81],
                'dgramrcvbuf_b' => $VPSID[$matrix_line][82],
                'dgramrcvbuf_l' => $VPSID[$matrix_line][83],
                'dgramrcvbuf_f' => $VPSID[$matrix_line][84],
                'numothersock' => $VPSID[$matrix_line][85],
                'numothersock_m' => $VPSID[$matrix_line][86],
                'numothersock_b' => $VPSID[$matrix_line][87],
                'numothersock_l' => $VPSID[$matrix_line][88],
                'numothersock_f' => $VPSID[$matrix_line][89],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'dcachesize' => $VPSID[$matrix_line][90],
                'dcachesize_m' => $VPSID[$matrix_line][91],
                'dcachesize_b' => $VPSID[$matrix_line][92],
                'dcachesize_l' => $VPSID[$matrix_line][93],
                'dcachesize_f' => $VPSID[$matrix_line][94],
                'numfile' => $VPSID[$matrix_line][95],
                'numfile_m' => $VPSID[$matrix_line][96],
                'numfile_b' => $VPSID[$matrix_line][97],
                'numfile_l' => $VPSID[$matrix_line][98],
                'numfile_f' => $VPSID[$matrix_line][99],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'numiptent' => $VPSID[$matrix_line][100],
                'numiptent_m' => $VPSID[$matrix_line][101],
                'numiptent_b' => $VPSID[$matrix_line][102],
                'numiptent_l' => $VPSID[$matrix_line][103],
                'numiptent_f' => $VPSID[$matrix_line][104],
                'diskspace' => $VPSID[$matrix_line][105],
                'diskspace_s' => $VPSID[$matrix_line][106],
                'diskspace_h' => $VPSID[$matrix_line][107],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'diskinodes' => $VPSID[$matrix_line][108],
                'diskinodes_s' => $VPSID[$matrix_line][109],
                'diskinodes_h' => $VPSID[$matrix_line][110],
                'laverage' => $VPSID[$matrix_line][111],
                'cpulimit' => $VPSID[$matrix_line][112],
                'cpuunits' => $VPSID[$matrix_line][113],
                'timestamp' => time()
                });

		if (!$VPSID[$matrix_line][114]) { $VPSID[$matrix_line][114] = "Yes" ; }
		if (!$VPSID[$matrix_line][115]) { $VPSID[$matrix_line][115] = '/vz/root/$VEID' ; }
		if (!$VPSID[$matrix_line][116]) { $VPSID[$matrix_line][116] = '/vz/private/$VEID' ; }

                ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'ONBOOT' => $VPSID[$matrix_line][114],
                'VE_ROOT' => $VPSID[$matrix_line][115],
                'VE_PRIVATE' => $VPSID[$matrix_line][116],
                'failcnt' => '0',
                'failcnt_lastevent' => time(),
                'failcnt_general' => '0',
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'OSTEMPLATE' => $VPSID[$matrix_line][117],
                'ORIGIN_SAMPLE' => $VPSID[$matrix_line][118],
                'NAMESERVER' => $VPSID[$matrix_line][119],
                'QUOTATIME' => $VPSID[$matrix_line][120],
                'DISK_QUOTA' => $VPSID[$matrix_line][121],
                'QUOTAUGIDLIMIT' => $VPSID[$matrix_line][122],
                'timestamp' => time()
                });

                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''

                'volume' => '/vz',
                'basedir' => 'private',
                #'suspend' => '0',  			# Not yet implemented.
                #'name' => 'admin', 			# Not written by the constructor. GUI handles this value.
                'timestamp' => time()
                });
	}

}


# Validate the settings before we pass them on to CCE:
sub validate {

    # ip:
    if (($VPSID[$matrix_line][1] == 0) && ($VPSID[$matrix_line][2] eq "running") ||
	($VPSID[$matrix_line][1] == 0) && ($VPSID[$matrix_line][2] eq "stopped") ||
	($VPSID[$matrix_line][1] == 0) && ($VPSID[$matrix_line][2] eq "mounted")) {

	# If we arrive at this point, then the entire vzlist maxtrix is shifted left from the 
	# 2nd column onwards. This may happen on new VPS which don't have an IP or hostname set yet. 
	# Means that all the data that we have gathered is junk. How to deal with this?
	# We have the vpsid, we have the tools (vzctl), so we set it manually and start over:
	$bad_info_detected++;
	system("/usr/sbin/vzctl set $currentvpsid --ipadd 127.0.0.1 --save");
    }

    if ($VPSID[$matrix_line][1] == 0) {
	$VPSID[$matrix_line][1] = "127.0.0.1";
    }
    # hostname:
    if ((!$VPSID[$matrix_line][2]) || ($VPSID[$matrix_line][2] eq "-")) {
	$VPSID[$matrix_line][2] = "localhost.localdomain";
    }
    # ostemplate:
    if (!$VPSID[$matrix_line][117]) {
	$VPSID[$matrix_line][117] = 'unknown';
    }

    ## If a VPS is stopped, then a LOT of values gathered from a VPS will be blanks, represented by a '-'. Which is an invalid
    ## character as far as our CCE-Schema is concerned. So we need to replace that with a '0'. But we cannot do that globally,
    ## as a VPS domain or hostname may also have a '-' in it. So we do that for each value independently:

    if ($VPSID[$matrix_line][3] ne "running") {

        $VPSID[$matrix_line][5] = "0";
        $VPSID[$matrix_line][6] = "0";
        $VPSID[$matrix_line][9] = "0";
        $VPSID[$matrix_line][10] = "0";
        $VPSID[$matrix_line][11] = "0";
        $VPSID[$matrix_line][14] = "0";
        $VPSID[$matrix_line][15] = "0";
        $VPSID[$matrix_line][16] = "0";
        $VPSID[$matrix_line][19] = "0";
        $VPSID[$matrix_line][20] = "0";
        $VPSID[$matrix_line][21] = "0";
        $VPSID[$matrix_line][24] = "0";
        $VPSID[$matrix_line][25] = "0";
        $VPSID[$matrix_line][26] = "0";
        $VPSID[$matrix_line][29] = "0";
        $VPSID[$matrix_line][30] = "0";
        $VPSID[$matrix_line][31] = "0";
        $VPSID[$matrix_line][34] = "0";
        $VPSID[$matrix_line][35] = "0";
        $VPSID[$matrix_line][36] = "0";
        $VPSID[$matrix_line][39] = "0";
        $VPSID[$matrix_line][40] = "0";
        $VPSID[$matrix_line][41] = "0";
        $VPSID[$matrix_line][44] = "0";
        $VPSID[$matrix_line][45] = "0";
        $VPSID[$matrix_line][46] = "0";
        $VPSID[$matrix_line][49] = "0";
        $VPSID[$matrix_line][50] = "0";
        $VPSID[$matrix_line][51] = "0";
        $VPSID[$matrix_line][54] = "0";
        $VPSID[$matrix_line][55] = "0";
        $VPSID[$matrix_line][56] = "0";
        $VPSID[$matrix_line][59] = "0";
        $VPSID[$matrix_line][60] = "0";
        $VPSID[$matrix_line][61] = "0";
        $VPSID[$matrix_line][64] = "0";
        $VPSID[$matrix_line][65] = "0";
        $VPSID[$matrix_line][66] = "0";
        $VPSID[$matrix_line][69] = "0";
        $VPSID[$matrix_line][70] = "0";
        $VPSID[$matrix_line][71] = "0";
        $VPSID[$matrix_line][74] = "0";
        $VPSID[$matrix_line][75] = "0";
        $VPSID[$matrix_line][76] = "0";
        $VPSID[$matrix_line][79] = "0";
        $VPSID[$matrix_line][80] = "0";
        $VPSID[$matrix_line][81] = "0";
        $VPSID[$matrix_line][84] = "0";
        $VPSID[$matrix_line][85] = "0";
        $VPSID[$matrix_line][86] = "0";
        $VPSID[$matrix_line][89] = "0";
        $VPSID[$matrix_line][90] = "0";
        $VPSID[$matrix_line][91] = "0";
        $VPSID[$matrix_line][94] = "0";
        $VPSID[$matrix_line][95] = "0";
        $VPSID[$matrix_line][96] = "0";
        $VPSID[$matrix_line][99] = "0";
        $VPSID[$matrix_line][100] = "0";
        $VPSID[$matrix_line][101] = "0";
        $VPSID[$matrix_line][104] = "0";
        $VPSID[$matrix_line][105] = "0";
        $VPSID[$matrix_line][108] = "0";
        $VPSID[$matrix_line][111] = "0";
        $VPSID[$matrix_line][123] = "0";
        $VPSID[$matrix_line][124] = "0";

    }
}

# Update or populate CCE with the CPU_POWER and CURRENT_CPU (usage) of the master node:
sub cce_update_cpu {

	# This is where we actually populate CCE with the gathered info:
        @oids = $cce->find('VserverMasterNode');
        if ($#oids < 0) {
                #print "Object not yet in CCE. Creating new one.\n";
                ($ok) = $cce->create('VserverMasterNode', {
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
                });
        }
        else {
                #print "Object already present in CCE. Updating it.\n";
                # find the system object
                ($sys_oid) = $cce->find('VserverMasterNode');
                ($ok, $sys) = $cce->get($sys_oid);
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
                });
	}
}

# Pull the existing sample VPS config files into CCE:
sub sampleconfigs {
    system ("rm -f /tmp/sampleconf");
    system("ls -k1 $vzcfgdir*.conf-sample | cut -d \/ -f5 |sed 's/ve-//'g | sed 's/.conf-sample//'g > /tmp/sampleconf");

    # Does the config file exist?
    $cfile = "/tmp/sampleconf";
    if (-e $cfile) {
        open(CFILE,"$cfile") || die("Could not find $cfile. $!");

        # Find all previous VserverConfigSamples entries and destroy them:
        @oldsamples = $cce->findx('VserverConfigSamples', { 'nusol' => 'nusol' });
        for $entry (@oldsamples) {
            ($ok) = $cce->destroy($entry);
        }

	# Parse the dumpfile:
        while ($line = <CFILE>) {
            chomp($line);
            next if $line =~ /^\s*$/; # skip blank lines
	    
	    # Populate CCE:
            ($ok) = $cce->create('VserverConfigSamples', {
            	    'nusol' => 'nusol',
            	    'ConfigShortName' => $line,
            	    'ConfigFullName' => 've-' . $line . '.conf-sample'
            	    });
        }
    }
    system ("rm -f /tmp/sampleconf");
}

# Pull the existing OS Templates into CCE:
sub ostemplates {
    if (!-d "/vz/template") {
	system("/bin/mkdir /vz/template");
    }
    if (!-d "/vz/template/cache") {
        system("/bin/mkdir /vz/template/cache");
    }
    if (!-f "/etc/vz/vz.conf") {

        if (!-d "/vz/cluster/") {
            system("/bin/mkdir /vz/cluster/");
        }
        if (!-d "/vz/cluster/etc") {
            system("/bin/mkdir /vz/cluster/etc");
        }
        if (!-d "/vz/cluster/etc/vz") {
            system("/bin/mkdir /vz/cluster/etc/vz");
        }
    
        if (!-d "/vz/cluster/etc/sysconfig") {
            system("/bin/mkdir /vz/cluster/etc/sysconfig");
        }
        if (!-d "/vz/cluster/etc/sysconfig/vz-scripts") {
            system("/bin/mkdir /vz/cluster/etc/sysconfig/vz-scripts");
        }
    }
    system ("rm -f /tmp/ostemplates");
    #system("ls -k1 /vz/template/cache/*.tar.gz |sed 's/\.tar\.gz//'g |sed 's/\/vz\/template\/cache\///'g > /tmp/ostemplates");
    system("ls -k1 /vz/template/cache/*.tar.gz |sed 's/\.tar\.gz//'g | cut -d / -f 5 > /tmp/ostemplates");

    # Does the dump file exist?
    $cfile = "/tmp/ostemplates";
    if (-e $cfile) {
        open(CFILE,"$cfile") || die("Could not find $cfile. $!");

        # Find all previous VserverOStemplates entries and destroy them:
        @oldsamples = $cce->findx('VserverOStemplates', { 'OSTnusol' => 'nusol' });
        for $entry (@oldsamples) {
            ($ok) = $cce->destroy($entry);
        }

	# Parse the dumpfile:
        while ($line = <CFILE>) {
            chomp($line);
            next if $line =~ /^\s*$/; # skip blank lines
	    
	    # Populate CCE:
            ($ok) = $cce->create('VserverOStemplates', {
            	    'OSTnusol' => 'nusol',
            	    'OStemplateShortName' => $line,
            	    'OStemplateFullName' => $line . '.tar.gz'
            	    });
        }
    }
    system ("rm -f /tmp/ostemplates");
}

# Handle IP address allocation:

sub ip_alloc {

    # Find existing vpsIPalloc objects:
    @old_alloc = $cce->findx('vpsIPalloc', {'vpsid' => $currentvpsid});
    if ($#old_alloc == -1) {
        # Nothing found in CCE
        if (@raw_ip_list) {

	    # But we found extra IPs in the config file
	    for $alloc_entry (@old_alloc) {
		# Destroy old allocation entries:
		($ok) = $cce->destroy($alloc_entry);
	    }

	    # Write new ones:
	    foreach $jummy_ip (@raw_ip_list) {
		if ($jummy_ip ne "") {
		    ($ok) = $cce->create('vpsIPalloc', {
        		'vpsid' => $currentvpsid,
               		'min' => $jummy_ip,
               		'creation_time' => time()
               	    });
		}
	    }
        }
    }
}

$cce->bye('SUCCESS');
exit(0);

# Legend of Matrix columns:

# 0 = vpsid
# 1 = ip
# 2 = hostname
# 3 = status
# 4 = numproc
# 5 = kmemsize
# 6 = kmemsize.m
# 7 = kmemsize.b
# 8 = kmemsize.l
# 9 = kmemsize.f
# 10 = lockedpages
# 11 = lockedpages.m
# 12 = lockedpages.b
# 13 = lockedpages.l
# 14 = lockedpages.f
# 15 = privvmpages
# 16 = privvmpages.m
# 17 = privvmpages.b
# 18 = privvmpages.l
# 19 = privvmpages.f
# 20 = shmpages
# 21 = shmpages.m
# 22 = shmpages.b
# 23 = shmpages.l
# 24 = shmpages.f
# 25 = numproc
# 26 = numproc.m
# 27 = numproc.b
# 28 = numproc.l
# 29 = numproc.f
# 30 = physpages
# 31 = physpages.m
# 32 = physpages.b
# 33 = physpages.l
# 34 = physpages.f
# 35 = vmguarpages
# 36 = vmguarpages.m
# 37 = vmguarpages.b
# 38 = vmguarpages.l
# 39 = vmguarpages.f
# 40 = oomguarpages
# 41 = oomguarpages.m
# 42 = oomguarpages.b
# 43 = oomguarpages.l
# 44 = oomguarpages.f
# 45 = numtcpsock
# 46 = numtcpsock.m
# 47 = numtcpsock.b
# 48 = numtcpsock.l
# 49 = numtcpsock.f
# 50 = numflock
# 51 = numflock.m
# 52 = numflock.b
# 53 = numflock.l
# 54 = numflock.f
# 55 = numpty
# 56 = numpty.m
# 57 = numpty.b
# 58 = numpty.l
# 59 = numpty.f
# 60 = numsiginfo
# 61 = numsiginfo.m
# 62 = numsiginfo.b
# 63 = numsiginfo.l
# 64 = numsiginfo.f
# 65 = tcpsndbuf
# 66 = tcpsndbuf.m
# 67 = tcpsndbuf.b
# 68 = tcpsndbuf.l
# 69 = tcpsndbuf.f
# 70 = tcprcvbuf
# 71 = tcprcvbuf.m
# 72 = tcprcvbuf.b
# 73 = tcprcvbuf.l
# 74 = tcprcvbuf.f
# 75 = othersockbuf
# 76 = othersockbuf.m
# 77 = othersockbuf.b
# 78 = othersockbuf.l
# 79 = othersockbuf.f
# 80 = dgramrcvbuf
# 81 = dgramrcvbuf.m
# 82 = dgramrcvbuf.b
# 83 = dgramrcvbuf.l
# 84 = dgramrcvbuf.f
# 85 = numothersock
# 86 = numothersock.m
# 87 = numothersock.b
# 88 = numothersock.l
# 89 = numothersock.f
# 90 = dcachesize
# 91 = dcachesize.m
# 92 = dcachesize.b
# 93 = dcachesize.l
# 94 = dcachesize.f
# 95 = numfile
# 96 = numfile.m
# 97 = numfile.b
# 98 = numfile.l
# 99 = numfile.f
# 100 = numiptent
# 101 = numiptent.m
# 102 = numiptent.b
# 103 = numiptent.l
# 104 = numiptent.f
# 105 = diskspace
# 106 = diskspace.s
# 107 = diskspace.h
# 108 = diskinodes
# 109 = diskinodes.s
# 110 = diskinodes.h
# 111 = laverage
# 112 = cpulimit
# 113 = cpuunits

### From config file:
# 114 = onboot
# 115 = ve_root
# 116 = ve_private
# 117 = ostemplate
# 118 = origin_sample
# 119 = nameserver
# 120 = quotatime
# 121 = disk_quota
# 122 = quotaugidlimit
# 123 = avnumproc_b
# 124 = avnumproc_l
# 125 = ip_list (IP address allocation)

