#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: vz_backup_vps_parse.pl, v1.1.0.0 Tue 11 Mar 2008 02:42:29 PM CET mstauber Exp $
# Copyright 2006-2008 Solarspeed Ltd. All rights reserved.

# This constructor parses /etc/vz/aconf/vps-vps-backup.conf and pushes the info into CCE.
# This is especially important for clusters on fail overs, so that the backup settings
# for individual VPS's are inherited on the new primary after a fail-over.

$whatami = "constructor";

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

if ($whatami eq "handler") {
    $cce->connectfd();
}
else {
    $cce->connectuds();
}

# Location of the config file:
$vserver_cronConfig = "/etc/vz/aconf/vps-vps-backup.conf";

# Config file present?
if (-e $vserver_cronConfig) {
	# Parse config file and shove their data into CCE:
	&main;
}

$cce->bye('SUCCESS');
exit(0);

##
# Subs:
##

sub main {

    # Pull the config file into a Matrix:
    #
    # I LOVE matrix stuff! If there is a sledload of data to parse, 
    # then there is hardly a more efficient way to do so.
    # A legend of what colums the matrix has is at the end of the code.

    open (F, $vserver_cronConfig) || die "Could not open $vserver_cronConfig: $!";
    $a = 0; 				# Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
    while ($line = <F>) {
	chomp($line);
	next if $line =~ /^\s*$/; 		# skip blank lines
	next if $line =~ /^#*$/; 		# skip if comment

	my (@row) = split (/&/, $line);
        $CONFIG[$a][0] = $row[0];
        $CONFIG[$a][1] = $row[1];
        $CONFIG[$a][2] = $row[2];
        $CONFIG[$a][3] = $row[3];
        $CONFIG[$a][4] = $row[4];
        $CONFIG[$a][5] = $row[5];
        $CONFIG[$a][6] = $row[6];
        $CONFIG[$a][7] = $row[7];
        $CONFIG[$a][8] = $row[8];
        $CONFIG[$a][9] = $row[9];
        $CONFIG[$a][10] = $row[10];

        if (($CONFIG[$a][0] ne "") && ($CONFIG[$a][1] ne "") && ($CONFIG[$a][2] ne "")) {
	    if ($DEBUG == "1") {
		print "--------------------------------------------------\n";
		print "vpsid : $CONFIG[$a][0] \n";
		print "ip : $CONFIG[$a][1] \n";
		print "hostname : $CONFIG[$a][2] \n";
		print "LBpresent : $CONFIG[$a][3] \n";
		print "LRpresent : $CONFIG[$a][4] \n";
		print "LRtimestamp : $CONFIG[$a][5] \n";
		print "LBtimestamp : $CONFIG[$a][6] \n";
		print "VserverBackupLocalEnabled : $CONFIG[$a][7] \n";
		print "VserverBackupRemoteEnabled : $CONFIG[$a][8] \n";
		print "VserverBackupEnabled : $CONFIG[$a][9] \n";
		print "VserverBackupRemoteAuthority : $CONFIG[$a][10] \n";
	    }
	    &feedthemonster;
        }
	$a++;
    }
    close(F);
}

# Subroutine that feeds the data into CCE:
sub feedthemonster {
    # This is where we actually populate CCE with the gathered info:
    @oids = $cce->find('Vserver_Backup', {'vpsid' => $CONFIG[$a][0]});
    if ($#oids < 0) {
	# Object not yet in CCE. Creating new one.
        ($ok) = $cce->create('Vserver_Backup', {
    	    'vpsid' => $CONFIG[$a][0],
    	    'ip' => $CONFIG[$a][1],
    	    'hostname' => $CONFIG[$a][2],
    	    'LBpresent' => $CONFIG[$a][3],
    	    'LRpresent' => $CONFIG[$a][4],
    	    'LRtimestamp' => $CONFIG[$a][5],
    	    'LBtimestamp' => $CONFIG[$a][6],
    	    'VserverBackupLocalEnabled' => $CONFIG[$a][7],
    	    'VserverBackupRemoteEnabled' => $CONFIG[$a][8],
    	    'VserverBackupEnabled' => $CONFIG[$a][9],
    	    'VserverBackupRemoteAuthority' => $CONFIG[$a][10]
        });
    }
    else {
        # Object already present in CCE. Updating it
        ($sys_oid) = $cce->find('Vserver_Backup', {'vpsid' => $CONFIG[$a][0]});
        ($ok, $sys) = $cce->get($sys_oid);
		
        ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
            'vpsid' => $CONFIG[$a][0],
            'ip' => $CONFIG[$a][1],
            'hostname' => $CONFIG[$a][2],
            'LBpresent' => $CONFIG[$a][3],
            'LRpresent' => $CONFIG[$a][4],
            'LRtimestamp' => $CONFIG[$a][5],
            'LBtimestamp' => $CONFIG[$a][6],
            'VserverBackupLocalEnabled' => $CONFIG[$a][7],
            'VserverBackupRemoteEnabled' => $CONFIG[$a][8],
            'VserverBackupEnabled' => $CONFIG[$a][9],
            'VserverBackupRemoteAuthority' => $CONFIG[$a][10]
        });
    }
}

